------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
------------------------------------------

local LrApplication = import 'LrApplication'
local LrBinding = import 'LrBinding'
local LrView = import 'LrView'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 

local myLogger = LrLogger('PhotomatixLogger')
myLogger:enable("print") 

local pmdebug=false

local cntPhotos=0
local folderPath=''

CreateHdrSettings = {}

function CreateHdrSettings.outputToLog(message)
	myLogger:trace(message)
end


function ConsistencyCheckSelectedPhotos()
	local checkStatus='ok' 
	local strErrMessage=''
	local iPhoto, wFirstPhoto, hFirstPhoto=0
	local activeCatalog = LrApplication.activeCatalog()

	activeCatalog:withReadAccessDo( function()

		local photos = activeCatalog.targetPhotos	
		cntPhotos=#photos

		for iPhoto=1, cntPhotos, 1 do		
			local wImage = photos[iPhoto]:getRawMetadata('width')
			local hImage = photos[iPhoto]:getRawMetadata('height')
			local fileSize = photos[iPhoto]:getRawMetadata('fileSize')
			local fileFormat = photos[iPhoto]:getRawMetadata('fileFormat')
			local fileName=photos[iPhoto]:getFormattedMetadata('fileName')
			local softwareTagPMP = photos[iPhoto]:getFormattedMetadata('software')
			local softwareTagPlugin=photos[iPhoto]:getPropertyForPlugin( _PLUGIN, 'software_tag' )
			if softwareTagPlugin then
				myLogger:trace(string.format('software tag set by Plugin: %s',softwareTagPlugin))
			end
			
			if not wImage or not hImage or not fileSize or (wImage<=0) or (hImage<=0) then
				return statusOK
			end
			myLogger:trace(string.format('%d x %d  -  fileSize: %d  is  %1.2f  times resolution',wImage,hImage,fileSize,fileSize/(wImage*hImage)))

			local isAlreadyMergedHDR=false
			if (fileFormat=='TIFF') then
				if ( softwareTagPlugin and (softwareTagPlugin=="Photomatix Merge To 32-bit HDR") ) then
					isAlreadyMergedHDR=true				
				elseif ( softwareTagPMP and (softwareTagPMP=="Photomatix Merge To 32-bit HDR") ) then
					isAlreadyMergedHDR=true
				elseif (fileSize>=(wImage*hImage*11))  then
					isAlreadyMergedHDR=true
				end
			end
			
			if isAlreadyMergedHDR then
				if not fileName then
					fileName=strings.format('# %d',iPhoto)
				end
				strErrMessage=string.format(LOC "$$$/Photomatix/General/Error/SelectedPhotoHdr=The file %s seems to be an already merged 32-bit TIFF file. Please deselect it.",fileName)
				LrDialogs.message(LOC "$$$/Photomatix/General/Error=Error", strErrMessage, "info")
				checkStatus='failed'							
				break
				
			else 		
				if (iPhoto==1) then
					wFirstPhoto=wImage
					hFirstPhoto=hImage
				else
					if ( (wImage~=wFirstPhoto) or (hImage~=hFirstPhoto)) then
						strErrMessage=LOC "$$$/Photomatix/General/Error/DimensionMisMatch=The photos you selected do not have the same dimension. Photo # 1 is  %d x %d  while photo # %d is  %d x %d."
						LrDialogs.message(LOC "$$$/Photomatix/General/Error=Error", 
							string.format(strErrMessage,wFirstPhoto,hFirstPhoto,iPhoto,wImage,hImage), 
							"info")				
						checkStatus='failed'				
						break
					end
				end
			end	
			if fileFormat then
				myLogger:trace(string.format('fileFormat:  %s',fileFormat))
			end
		end
	end)
	return checkStatus
end


function CreateHdrSettings.debutCommun(fic1, fic2, minSeuilLen)
	local resStr=''
	local i=1
	local lenFicMin=#fic1
	if #fic2<lenFicMin then 
		lenFicMin=#fic2
	end
	while (i<=lenFicMin) do
		if string.sub(fic1, i, i) ~= string.sub(fic2, i, i) then
			break
		end
		i=i+1
	end	
	if (i>minSeuilLen) then
		resStr=string.sub(fic1, 1, i-1)
	end
	return resStr
end

function CreateHdrSettings.firstFileName( )
	local firstFileName=''
	local activeCatalog = LrApplication.activeCatalog()
	activeCatalog:withReadAccessDo( function()
		local photos = activeCatalog.targetPhotos	
		firstFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[1].path)) 
	end )
	return firstFileName
end


function CreateHdrSettings.defaultFileName( )
	local combinedFileName=''
	
	if prefs.LrVersion<2 then
		return combinedFileName
	end
	local activeCatalog = LrApplication.activeCatalog()
	activeCatalog:withReadAccessDo( function()
		local photos = activeCatalog.targetPhotos	

		cntPhotos=#photos  --		CreateHdrSettings.outputToLog(string.format('cntPhotos: %d',cntPhotos))
		folderPath=LrPathUtils.parent(photos[1].path)  

		local iPhoto=1
		local commonStr=''
		local firstFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[1].path))  ---		CreateHdrSettings.outputToLog(string.format('firstFileName%d:  %s',iPhoto,firstFileName))
		local maxLen=0
		for iPhoto=2, cntPhotos, 1 do		
			local curFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[iPhoto].path))   --			CreateHdrSettings.outputToLog(string.format('curPath%d:  %s',iPhoto,photos[iPhoto].path))
			if (iPhoto==2) then
				commonStr=CreateHdrSettings.debutCommun(firstFileName, curFileName, 2) 
			else 
				if #commonStr>1 then
					commonStr=CreateHdrSettings.debutCommun(commonStr, curFileName, 2) 
				end
			end	
			if #curFileName>maxLen then 
				maxLen=#curFileName
			end
		end

		if #commonStr>1 then
			if ((maxLen - #commonStr) <= 8) then
				combinedFileName=firstFileName
				local cntSameName=0
				for iPhoto=2, cntPhotos, 1 do		
					local curFileName=LrPathUtils.leafName(LrPathUtils.removeExtension(photos[iPhoto].path))
					local differingPart=string.sub(curFileName, #commonStr+1)
					if #differingPart>0 then
						combinedFileName=string.format('%s_%s', combinedFileName, differingPart)  --					CreateHdrSettings.outputToLog(string.format('combinedFileName%d:  %s',iPhoto,combinedFileName))
					else
						cntSameName=cntSameName+1
					end		
				end
				if (cntSameName==(cntPhotos-1)) then
					combinedFileName=string.format('%sx%d',firstFileName,cntPhotos)
				end
			end
		end	
		if ((#combinedFileName<2) or (cntPhotos > 7)) then
			combinedFileName=string.format('%sAnd%dmore', firstFileName, cntPhotos-1)
		end
		
	end )

	return combinedFileName
end


function CreateHdrSettings.getOptions( )

	if prefs.LrVersion==nil then
		LrDialogs.message("Plugin did not start correctly","Please check whether you invoked the plugin by choosing 'File > Plug-in Extras'","info")
		return
	end

	prefs.processType='none'

	LrFunctionContext.callWithContext( 'CreateHdrSettings', function( context )

	local f = LrView.osFactory() 
	local props = LrBinding.makePropertyTable( context ) 
	props.chkAlign=true
	props.alignType='1'
	props.chkCrop=true
	props.chkPerspective=true
	props.chkReduceNoise=false
	props.nrWhat='0'
	props.chkGhosting=false
	props.ghostingType='1'
	props.ghostingDetection='normal'
	props.stackWith=false
	props.saveAsHalf=false
	props.needNormalise=true
	props.chkAddSuffix=false

	props.fileNameSuffix=''
	props.outputFileNameOption='0'
	props.combinedFileName=CreateHdrSettings.defaultFileName()  --	CreateHdrSettings.outputToLog(string.format('props.outputName:  %s',props.outputFileName))
	props.firstFileName=CreateHdrSettings.firstFileName()  

	if (prefs.chkAlign~=nil) then
		props.chkAlign=prefs.chkAlign
	end
	if (prefs.chkCrop~=nil) then
		props.chkCrop=prefs.chkCrop
	end
	if (prefs.alignType~=nil) then
		if ((prefs.alignType=='1')or(prefs.alignType=='2')) then
			props.alignType=prefs.alignType
		end
	end
	if (prefs.chkPerspective~=nil) then
		props.chkPerspective=prefs.chkPerspective
	end
	if (prefs.ghostingType~=nil) then
	        if ((prefs.ghostingType=='1') or (prefs.ghostingType=='2')) then
		        props.ghostingType=prefs.ghostingType
		end
	end
	if ((prefs.chkGhosting)and((prefs.ghostingType=='1')or(prefs.ghostingType=='2'))) then
		props.chkGhosting=prefs.chkGhosting
	end
	if (prefs.ghostingDetection~=nil) then
		props.ghostingDetection=prefs.ghostingDetection
	end
	if (prefs.chkReduceNoise) then
		props.chkReduceNoise=prefs.chkReduceNoise
	end
	if (prefs.nrWhat~=nil) then
		props.nrWhat=prefs.nrWhat
	end
	if (prefs.stackWith) then
		props.stackWith=prefs.stackWith
	end
	if (prefs.saveAsHalf~=nil) then
		props.saveAsHalf=prefs.saveAsHalf
	end
	if (prefs.needNormalise~=nil) then
		props.needNormalise=prefs.needNormalise
	end
	if (prefs.outputFileNameOption~=nil) then
		props.outputFileNameOption=prefs.outputFileNameOption
	end
	if (prefs.chkAddSuffix) then
		props.chkAddSuffix=prefs.chkAddSuffix
	end
	if (prefs.fileNameSuffix~=nil) then
		props.fileNameSuffix=prefs.fileNameSuffix
	end

	local updateFileNameFunction = function() 
		if props.outputFileNameOption == '10' then
			return
		end
		local curFileName=props.combinedFileName
		if props.outputFileNameOption == '1' then
			curFileName=props.firstFileName
		end
		outputFileNameEditField.value=string.format('%s%s', curFileName, props.fileNameSuffix)
	end
	props:addObserver( "fileNameSuffix", updateFileNameFunction )
	
	local LeftMarginSavePanel=135
	
	local outputFileNameEditField=f:edit_field { 
		width_in_chars = 24, 
		value = LrView.bind {
			keys = { 'outputFileNameOption', 'chkAddSuffix', 'fileNameSuffix' },
				operation = function( binder, values, fromTable )
					local curFileName=''
					if fromTable then
						if values.outputFileNameOption == '0' then
							curFileName=props.combinedFileName
						end
						if values.outputFileNameOption == '1' then
							curFileName=props.firstFileName
						end
						if values.outputFileNameOption == '10' then
							curFileName=''
						else
							if values.chkAddSuffix and props.fileNameSuffix~=nil then
								curFileName=string.format('%s%s', curFileName, props.fileNameSuffix)
							end
						end
						return curFileName
				   end
				   return LrBinding.kUnsupportedDirection
				end 

		},
		immediate = true,
		enabled = LrView.bind {
			key = "outputFileNameOption", 
			transform = function(value, fromTable)
			return value == '10' end
		},
	}
	

	local multipleFilesSettings = f:column {
		bind_to_object = props,
		spacing = f:dialog_spacing(),
		
		f:row {
			f:static_text {
				title = LOC ("$$$/Photomatix/Settings/Heading=You have selected ^1 photos to be merged to HDR by Photomatix Pro ^nand saved as a Floating Point TIFF file in 32 Bits/Channel mode.^nPlease set the options for merging and saving.",cntPhotos),
				warps=true,
				alignment = 'left',
			},
		},

		f:group_box {
			title = LOC "$$$/Photomatix/Settings/CreateHDR/Title=Pre-processing and Merging",
			show_title=true,
			fill_horizontal=1,
			font = "<system>",

		f:row {	
			spacing = f:control_spacing() / 2,
			f:column {
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/Align/Checkbox=Align images                      ",
					tooltip=LOC "$$$/Photomatix/Settings/Align/Tooltip=Corrects misalignment problems when the camera moves slightly between the bracketed frames.  Always check this option when your bracketed photos are taken hand-held, but do not check it if they are stitched panoramas or derived from a single Raw file.",
					value = LrView.bind ("chkAlign"),
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:column {
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/Crop/Checkbox=Crop aligned result",
					tooltip=LOC "$$$/Photomatix/Settings/Crop/Tooltip=Checking this option removes the unneeded borders resulting from the alignment adjustments.  Unchecking it disables this, so that the resulting image has exactly the same width and height as the source images.",
					value = LrView.bind ("chkCrop"),
					enabled = LrView.bind("chkAlign"),
					checked_value = true,
					unchecked_value = false,
				},
			},
		},

		f:row {	
			f:column {
				spacing = f:control_spacing() *2/3,
				margin_left = 15,
				f:radio_button {
					title =LOC "$$$/Photomatix/Settings/AlignMethod1=by correcting horizontal and vertical shifts",
					tooltip=LOC "$$$/Photomatix/Settings/AlignMethod1/Tooltip=This alignment method is fast, but only corrects for translation movements. Recommended for images shot on a tripod.",
					value = LrView.bind ("alignType"),
					checked_value = '1',
					enabled = LrView.bind("chkAlign"),
				},
				f:row {	
				f:column {
					f:radio_button {
						title =LOC "$$$/Photomatix/Settings/AlignMethod2=by matching features",
						tooltip=LOC "$$$/Photomatix/Settings/AlignMethod2/Tooltip=This alignment method corrects for rotation, translation and scaling differences, as well as perspective distortion if included.",
						value = LrView.bind ("alignType"),
						checked_value = '2',
						enabled = LrView.bind("chkAlign"),
					},
				},
				f:column {
					f:checkbox {
						title = LOC "$$$/Photomatix/Settings/PerspectiveCorrection/Checkbox=include perspective correction",
						tooltip=LOC "$$$/Photomatix/Settings/PerspectiveCorrection/Tooltip=When checked, requests that the alignment by matching features corrects for perspective distortion in addition to correcting for translation, rotation and scaling differences.",
						value = LrView.bind ("chkPerspective"),
						checked_value = true,
						unchecked_value = false,
						enabled = LrView.bind {
							keys = { 'chkAlign', 'alignType' },
							operation = function( binder, values, fromTable )
							   if fromTable then
							      return values.chkAlign and values.alignType == '2'
							   end
							   return LrBinding.kUnsupportedDirection
							end }
					},
				},
				},
			},				
		},


		f:row {	
			spacing = f:control_spacing() / 3,

			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/Deghost=Remove ghosts",
				tooltip=LOC "$$$/Photomatix/Settings/Deghost/Tooltip=Only check this option if your scene includes elements that moved between the bracketed shots. Otherwise, leave it unchecked, as removing ghosts lowers the quality of the merged image.",
				value = LrView.bind ("chkGhosting"),
				checked_value = true,
				unchecked_value = false,
			},
		},

		f:row {	
			spacing = f:control_spacing() / 2,
			f:column {			
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/NR=Reduce noise",
					tooltip=LOC "$$$/Photomatix/Settings/NR/Tooltip=High quality noise reduction but significantly increases processing times. It is better to disable Lightroom Noise Reduction when you check this option",
					value = LrView.bind ("chkReduceNoise"),
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:column {			
				f:popup_menu {
					value = LrView.bind ("nrWhat"),
					items = {
						{ title = LOC "$$$/Photomatix/Settings/NR/UnderOnly=Underexposed image(s) only", value = "0" },
						{ title = LOC "$$$/Photomatix/Settings/NR/NormalUnder=Normal exposure & underexposed", value = "1" },
						{ title = LOC "$$$/Photomatix/Settings/NR/All=All source images", value = "2" },
					},
					fill_horizontal=0.4,
					enabled = LrView.bind("chkReduceNoise"),
				},
			},
		},


		f:row {					
			spacing = f:control_spacing(),
		},
		
		},
		
		f:group_box {
			title = LOC "$$$/Photomatix/Settings/BoxTitle=Saving",
			fill_horizontal=1,
			show_title=true,
			font = "<system>",

		f:row {	
--			spacing = f:control_spacing(),
			f:column {		
				f:static_text {
					title = LOC "$$$/Photomatix/Settings/OutputFileNameLabel=Name Merged File:",
					width=LeftMarginSavePanel-5,
					alignment = 'right',
				},			
			},
			f:column {			
				f:popup_menu {
					value = LrView.bind ("outputFileNameOption"),
					items = {
						{ title = LOC "$$$/Photomatix/Settings/OutputFileName/CombinedFileName=Combined file names", value = "0" },
						{ title = LOC "$$$/Photomatix/Settings/OutputFileName/FileNameFirstPhoto=File name of first photo                       ", value = "1" },
						{ separator = true}, 
						{ title = LOC "$$$/Photomatix/Settings/OutputFileName/CustomFileName=Custom file name...", value = "10" },
					},
				},
			},			
		},		
		f:row {	
			margin_left = LeftMarginSavePanel,
			outputFileNameEditField,
		},	

		f:row {	

			f:column {
				f:static_text {
					title = LOC "$$$/NotToBeEditedByTranslators/SpaceForFitting/LeftMarginAddSuffix=        ",
				},			
			},
			
			f:column {
				f:checkbox {
					title = LOC "$$$/Photomatix/Settings/OutputFileName/SuffixLabel=Add suffix:",
					value = LrView.bind ("chkAddSuffix"),
					checked_value = true,
					unchecked_value = false,
					enabled = LrView.bind {
						key = "outputFileNameOption", 
						transform = function(value, fromTable)
						return value ~= '10' end
					},
				},
			},

			f:column {
				f:edit_field { 
					width_in_chars = 15, 
					value = LrView.bind ("fileNameSuffix"), 
					enabled = LrView.bind {
						keys = { 'outputFileNameOption', 'chkAddSuffix' },
						operation = function( binder, values, fromTable )
							local curFileName=''
							if fromTable then
								return values.chkAddSuffix and values.outputFileNameOption ~= '10' 
						   end
						   return LrBinding.kUnsupportedDirection
						end 
					}	

				},
			},
		},


		f:row {	
			margin_left = LeftMarginSavePanel,
			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/Stack/Multiple=Stack with first selected photo",
				value = LrView.bind ("stackWith"), 
			},			
		},

		f:row {	
			margin_left = LeftMarginSavePanel,
			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/SaveAsHalf=Use Half Floating Point format",
				tooltip=LOC "$$$/Photomatix/Settings/Normalise/Tooltip=Reduces output file size by half while encoding the same dynamic range as the OpenEXR HDR file format. Recommended to save disk space.",
				value = LrView.bind ("saveAsHalf"), 
			},			
		},

		f:row {	
			margin_left = LeftMarginSavePanel,
			f:checkbox {
				title = LOC "$$$/Photomatix/Settings/Normalise=Scale pixel values to fixed range",
				tooltip=LOC "$$$/Photomatix/Settings/Normalise/Tooltip=Checking this option avoids bright scenes appearing too bright and dark scenes too dark when viewed at Lightroom's default exposure.",
				value = LrView.bind ("needNormalise"), 
			},			
		},


		},
	}

	if cntPhotos==1 then
		LrDialogs.message(LOC "$$$/Photomatix/General/Note=Note", 
			LOC "$$$/Photomatix/General/SinglePhotoMerge=You selected a single photo.  To create a 32-bit HDR file, please select bracketed photos, i.e. photos of the same scene taken under different exposures.", "info")

	elseif (ConsistencyCheckSelectedPhotos()=="ok") then
		
		local result = LrDialogs.presentModalDialog( {
			title = LOC "$$$/Photomatix/CreateHdrSettings/Title=Merge to 32-bit HDR",
			contents = multipleFilesSettings,
			actionVerb = LOC "$$$/Photomatix/General/CreateHDR=Merge",
		} )

		if pmdebug then
		CreateHdrSettings.outputToLog(string.format('result dialog:  %s',result))
		CreateHdrSettings.outputToLog(string.format('FileNameOption: %d   -  FileNameValue:  %s',props.outputFileNameOption, outputFileNameEditField.value))
		end
		
		local outputFileName=props.combinedFileName
		if props.outputFileNameOption == '10' then
			outputFileName=outputFileNameEditField.value
			if outputFileName == '' then
				outputFileName=props.combinedFileName
			end
		else
			if props.outputFileNameOption == '1' then
				outputFileName=props.firstFileName
			end
			if props.chkAddSuffix and props.fileNameSuffix~=nil then
				outputFileName=string.format('%s%s', outputFileName, props.fileNameSuffix)
			end
		end
			
		if pmdebug then
			CreateHdrSettings.outputToLog(string.format('OutputFileName:  %s',outputFileName))
		end

		if (result=='ok') then
			prefs.processType='CreateHDR'
			prefs.chkAlign=props.chkAlign
			prefs.chkCrop=props.chkCrop
			prefs.alignType=props.alignType
			prefs.chkPerspective=props.chkPerspective
			prefs.chkGhosting=props.chkGhosting
			prefs.ghostingType=props.ghostingType
			prefs.ghostingDetection=props.ghostingDetection
			prefs.chkReduceNoise=props.chkReduceNoise
			prefs.nrWhat=props.nrWhat
			prefs.stackWith=props.stackWith
			prefs.outputFormat="tif32"
			if props.saveAsHalf then
				prefs.outputFormat="tif16"
			end	
			prefs.saveAsHalf=props.saveAsHalf
			prefs.needNormalise=props.needNormalise
			prefs.outputFileNameOption=props.outputFileNameOption
			prefs.chkAddSuffix=props.chkAddSuffix
			prefs.fileNameSuffix=props.fileNameSuffix
			
			local outputExt=string.sub(prefs.outputFormat, 1, 3)
			
			prefs.outputPath=LrPathUtils.child(folderPath, LrPathUtils.addExtension(outputFileName, outputExt))
			prefs.outputPath=LrFileUtils.chooseUniqueFileName(prefs.outputPath)
		end	
	
	end
end )

end
